/*
m_chord_play_midi - Generate a tri- or tetra chord (output MIDI notes)

DESCRIPTION
Output the MIDI notes of a chord with three or four notes. Chords are
predefined in a function table, they are chosen by index, 0-42 in version 1.0.
The notes are output in an array. For trichords the last note will be -1 and
should be ignored.
The name of a chord corresponding to an index can be retrieved by using
m_chord_get_name .

SYNTAX
kChord[] m_chord_play_midi kChordIndex, kRootNote, kInversion [, kLimit]

INITIALIZATION

PERFORMANCE
kChord[] - A four element array holding the chord. A note value of -1
	should be ignored
kChordIndex - Index of the chord to play, corresponding names can be retrieved
	by using m_chord_get_name, internally limited to 0-number of chords -1
kRootNote - MIDI Root note of the chord, internally limited to 0-127
kInversion - Inversion of the chord, between 0 and 3, internally limited
	to that range
kLimit - What to do with notes out of the 0-127 range (default 0)
	0 = pass, notes will be output regardless
	1 = limit, limit notes to 0-127 range
	2 = invalidate, set out of range notes to -1

CREDITS
Author: Jeanette C.
*/

opcode m_chord_play_midi, k[], kkkO
	kChordIndex, kRootNote, kInversion, kLimit xin

	; Initialise the output array
	kNotes[] = fillarray(-1, -1, -1, -1)

	; Total number of known chords
	iNumberOfChords = ftlen(giChord_chords) / 3

	; Integral chord index, root note and inversion
	kIntIndex = limit:k(int(kChordIndex), 0, (iNumberOfChords -.9))
	kIntRoot = limit:k(round(kRootNote), 0, 127.1)
	kIntInversion = limit:k(round(kInversion), 0, 3.1)
	kIntLimit = limit:k(round(kLimit), 0, 2.1)

	; Internal index into the chord to be processed
	kNoteIndex init 0
	kTmpNote init -1

	; Only calculate a chord when an input changes
	if (changed(kIntIndex, kIntRoot, kIntInversion) != 0) then
		Sname = m_chord_get_name(kIntIndex)
		; Retrieve the chord and add root note
		kNoteIndex = 0
		kNotes[0] = kIntRoot ; always the root note
		while (kNoteIndex < 3) do
			kTmpNote = tab(((kIntIndex * 3) + kNoteIndex), giChord_chords, 0)
			if (kTmpNote != -1) then
				kNotes[kNoteIndex + 1] = (kTmpNote + kIntRoot)
			else
				kNotes[kNoteIndex + 1] = -1
			endif
			kNoteIndex += 1
		od

		; Process inversion
		if (kIntInversion >= 1) then
			kNotes[0] = kNotes[0] + 12
		endif

		if (kIntInversion >= 2) then
			kNotes[1] = kNotes[1] + 12
		endif

		if (kIntInversion >= 2) then
			kNotes[2] = kNotes[2] + 12
		endif

		if (kIntInversion >= 3) then
			; Different cases for tri and tetra chords
			if (kNotes[3] == -1) then
				kNotes[0] = kNotes[0] - 12
			else ; a four note chord
				kNotes[3] = kNotes[3] + 12
			endif
		endif ; third inversion

		; Limit if necessary
		if (kLimit == 1) then ; limit out-of-range notes to 0-127
			kNoteIndex = 0
			while (kNoteIndex <4) do
				kNotes[kNoteIndex] = limit:k(kNotes[kNoteIndex], 0, 127.1)
				kNoteIndex += 1
			od
		elseif (kLimit == 2) then ; set out-of-range notes to -1
			kNoteIndex = 0
			while (kNoteIndex < 4) do
				if ((kNotes[kNoteIndex] < 0) || (kNotes[kNoteIndex] > 127)) then
					kNotes[kNoteIndex] = -1
				endif
				kNoteIndex += 1
			od
		endif ; note limiting
	endif ; chord calculation

	; Output the note
	xout(kNotes)
endop
